/**********************************************************************
SQL Server Internals from the Practical Angle

Dmitri Korotkevitch
http://aboutsqlserver.com
dmitri@aboutsqlserver.com

Locking and Blocking: Application Locks (Session 1)
**********************************************************************/

use TroubleshootingDb
go

set nocount on
go

if exists
(
	select *
	from sys.procedures p join sys.schemas s on 
		p.schema_id = s.schema_id
	where
		s.name = 'dbo' and p.name = 'SerializeLoad'
)
	drop proc dbo.SerializeLoad
go


if exists(
	select *
	from sys.tables t join sys.schemas s on
		t.schema_id = s.schema_id
	where
		t.name = 'ProcessingQueue' and s.name = 'dbo'
)
	drop table dbo.ProcessingQueue
go

create table dbo.ProcessingQueue
(
	ID int not null,
	Data char(100) null,
	ProcessingTime datetime2(0),

	constraint PK_ProcessingQueue
	primary key clustered(ID)
)
go

;with CTE(IntVal)
as
(
	select 1
	
	union all
	
	select IntVal + 1
	from CTE
	where IntVal < 10000
)
insert into dbo.ProcessingQueue(ID)
	select IntVal
	from CTE
option (maxrecursion 0)
go


create proc dbo.SerializeLoad
as
begin
	set xact_abort on
	set nocount on
	
	declare
		@ResCode int
		,@MaxTime datetime2(0)

	create table #Result  
	(
		ID int not null primary key,
		Data char(100) null 
	)

	begin tran
		exec @ResCode = sys.sp_getapplock
			@Resource = N'MyAppLock',
			@LockMode = 'EXCLUSIVE',
			@LockTimeout = 30000 -- 30 seconds

		if @ResCode >= 0
		begin	
			-- Success
			insert into #Result(ID,Data)
				select top 100 ID, Data 
				from dbo.ProcessingQueue
				where 
					ProcessingTime is null or 
					ProcessingTime < dateadd(minute,-10,getdate())

			if @@ROWCOUNT > 0
				update dbo.ProcessingQueue
				set ProcessingTime = getdate()
				where ID in (select ID from #Result)
		end
		waitfor delay '00:00:15.000'      
	commit
	select ID, Data from #Result
end
go
		    
exec dbo.SerializeLoad