/**********************************************************************
SQL Server Internals from the Practical Angle

Dmitri Korotkevitch
http://aboutsqlserver.com
dmitri@aboutsqlserver.com

Locking and Blocking: Deadlocks due multiple updates (Session 1)
**********************************************************************/

use TroubleshootingDb
go

set nocount on
go

use TroubleshootingDb
go

set nocount on
go

create table dbo.Users
(
	UserId int not null,
	ExternalId int not null, 
	RawData1 int null,
	RawData1Processed int null,
	RawData2 int null,
	ModifiedTime datetime null,
	Placeholder char(200) not null
		constraint DEF_Users_Placeholder
		default 'a',
		
	constraint PK_Users
	primary key clustered(UserId)
)	
go

create unique index IDX_Users_ExternalID
on dbo.Users(ExternalId)
go

;with CTE(Id)
as
(
	select 1
	
	union all
	
	select Id + 1
	from CTE
	where Id < 1000
)
insert into dbo.Users(UserId, ExternalId)
	select Id, Id
	from CTE
option (MAXRECURSION 0)
go

-- SESSION 1 Code
-- Step 1
set transaction isolation level read committed
begin tran
	update dbo.Users 
	set 
		RawData1 = 1
	where 
		ExternalId = 1
		
	-- Do some processing here
	-- Step 2
	update dbo.Users 
	set 
		RawData1Processed = 10
	where 
		ExternalId = 1
commit
go





-- Ok, but what about this pattern?

-- Now let's create the trigger
create trigger dbo.trgUsersUpdate
on dbo.Users
after update
as
begin
	if @@ROWCOUNT = 0
		return
	
	if UPDATE(RawData1)
	begin	
		-- let's emulate some activity
		waitfor delay '0:00:05.000'
		
		update dbo.Users
		set ModifiedTime = GETDATE()
		where ExternalId in 
			(
				select ExternalId
				from inserted
			)
	end
end
go


set transaction isolation level read committed
begin tran
	update dbo.Users 
	set 
		RawData1 = 1
	where 
		ExternalId = 1
commit
go
